require 'base64'

class Linkificator::ConditionsStorage

  attr_reader :conditions, :options

  def initialize(conditions, options={})
    @conditions = conditions.map { |hash| Condition.new(hash.merge(options)) }
    @options    = options
  end

  def condition_for(options={})
    @conditions.detect do |condition|
      condition if condition.match?(options)
    end
  end

  def link_to(name = nil, options = nil, html_options = nil, &block)
    context = extract_context(name, options, html_options, &block)
    condition = condition_for(context) || Condition.new(self.options)
    condition.link_to(name, options, html_options, &block)
  end

private

  def extract_context(name = nil, options = nil, html_options = nil, &block)
    html_options, options = options, name if block_given?
    context = {}
    context[:target] = options if options.is_a?(String)
    context.merge!((html_options || {}).delete(:context) || {})
  end
end
